# Generated by Plain 0.124.1 on 2026-03-19 22:26

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "plainjobs",
            "0007_remove_jobrequest_plainjobs_jobrequest_unique_job_class_key_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="jobrequest",
            options={"ordering": ["-priority", "-created_at"]},
        ),
    ]
