# Generated by Plain 0.74.0 on 2025-10-10 20:57

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0005_rename_constraints_and_indexes"),
    ]

    operations = [
        migrations.AlterModelTable(
            name="jobprocess",
            table=None,
        ),
        migrations.AlterModelTable(
            name="jobrequest",
            table=None,
        ),
        migrations.AlterModelTable(
            name="jobresult",
            table=None,
        ),
    ]
