# Generated by Plain 0.63.0 on 2025-09-15 21:29

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0002_job_span_id_job_trace_id_jobrequest_span_id_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Job",
            new_name="JobProcess",
        ),
        migrations.RenameField(
            model_name="jobresult",
            old_name="job_uuid",
            new_name="job_process_uuid",
        ),
        # To fix subsequent migrations...
        migrations.AlterModelTable(
            name="JobRequest",
            table="plainworker_jobrequest",
        ),
        migrations.AlterModelTable(
            name="JobProcess",
            table="plainworker_jobprocess",
        ),
        migrations.AlterModelTable(
            name="JobResult",
            table="plainworker_jobresult",
        ),
    ]
