# Generated by Plain 0.127.2 on 2026-03-25 20:19

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0009_remove_jobprocess_plainjobs_j_job_cla_19f3c1_idx_and_more"),
    ]

    operations = []
