# Generated by Plain 0.85.0 on 2025-11-09 03:07

from plain import postgres
from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0006_alter_jobprocess_table_alter_jobrequest_table_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="jobprocess",
            old_name="unique_key",
            new_name="concurrency_key",
        ),
        migrations.RenameField(
            model_name="jobrequest",
            old_name="unique_key",
            new_name="concurrency_key",
        ),
        migrations.RenameField(
            model_name="jobresult",
            old_name="unique_key",
            new_name="concurrency_key",
        ),
        migrations.AlterField(
            model_name="jobprocess",
            name="priority",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobprocess",
            name="retries",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobprocess",
            name="retry_attempt",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobrequest",
            name="priority",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobrequest",
            name="retries",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobrequest",
            name="retry_attempt",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobresult",
            name="priority",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobresult",
            name="retries",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobresult",
            name="retry_attempt",
            field=postgres.SmallIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name="jobresult",
            name="status",
            field=postgres.TextField(
                choices=[
                    ("SUCCESSFUL", "Successful"),
                    ("ERRORED", "Errored"),
                    ("CANCELLED", "Cancelled"),
                    ("DEFERRED", "Deferred"),
                    ("LOST", "Lost"),
                ],
                max_length=20,
            ),
        ),
    ]
